------------------------------------------------------------------------------
--
--  package Kalender (body)
--
--  Tilbehr til pakken Ada.Calendar.
--
------------------------------------------------------------------------------
--  Update information:
--
--  1997.08.02-06 (Jacob Sparre Andersen)
--    Written.
--    The implementation of Date_Image and Time_Image was copied from CGI.Log
--      (1997.02.19).
--
--  (Insert additional update information above this line.)
------------------------------------------------------------------------------

------------------------------------------------------------------------------
--  Standardbiblioteker:

with Ada.Characters.Handling;
with Ada.Strings.Fixed;

------------------------------------------------------------------------------
--  Egne biblioteker:

with Zero_Filled_Image;

------------------------------------------------------------------------------

package body Kalender is

   ---------------------------------------------------------------------------
   --  function Image_2 (private):

   function Image_2 is new Zero_Filled_Image (Num => Integer, Width => 2);

   ---------------------------------------------------------------------------
   --  function Image_4 (private):

   function Image_4 is new Zero_Filled_Image (Num => Integer, Width => 4);

   ---------------------------------------------------------------------------  
   --  function Date_Image:

   function Date_Image (Item : in     Ada.Calendar.Time) return String is

      use Ada.Calendar;

   begin --  Date_Image
      return Image_4 (Year (Item)) & "-" & Image_2 (Month (Item)) & "-" &
             Image_2 (Day (Item));
   end Date_Image;

   ---------------------------------------------------------------------------  
   --  function Date_Value:

   function Date_Value (Item : in     String) return Ada.Calendar.Time is

      ---------------------------------------------------------------------
      --  Digit_Values:

      Digit_Values : constant array (Character range '0' .. '9') of Integer
                       := (0, 1, 2, 3, 4, 5, 6, 7, 8, 9);

      ---------------------------------------------------------------------

      use Ada.Calendar;
      use Ada.Characters.Handling;

      Year, Month, Day : Natural := 0;
      Index            : Positive := Item'First;

   begin --  Date_Value

   Skip_Spaces:
      loop
         exit Skip_Spaces when Index > Item'Last;
         exit Skip_Spaces when Item (Index) /= ' ';

         Index := Index + 1;
      end loop Skip_Spaces;

   Read_Year:
      loop
         exit Read_Year when Index > Item'Last;
         exit Read_Year when not Is_Digit (Item (Index));

         Year := Year * 10 + Digit_Values (Item (Index));
         Index := Index + 1;
      end loop Read_Year;

   Skip_First:
      loop
         exit Skip_First when Index > Item'Last;
         exit Skip_First when Is_Digit (Item (Index));

         Index := Index + 1;
      end loop Skip_First;

   Read_Month:
      loop
         exit Read_Month when Index > Item'Last;
         exit Read_Month when not Is_Digit (Item (Index));

         Month := Month * 10 + Digit_Values (Item (Index));
         Index := Index + 1;
      end loop Read_Month;

   Skip_Second:
      loop
         exit Skip_Second when Index > Item'Last;
         exit Skip_Second when Is_Digit (Item (Index));

         Index := Index + 1;
      end loop Skip_Second;

   Read_Day:
      loop
         exit Read_Day when Index > Item'Last;
         exit Read_Day when not Is_Digit (Item (Index));

         Day := Day * 10 + Digit_Values (Item (Index));
         Index := Index + 1;
      end loop Read_Day;

      return Time_Of (Year, Month, Day);
   end Date_Value;

   ---------------------------------------------------------------------------
   --  function Time_Image:

   function Time_Image (Item : in     Ada.Calendar.Time) return String is

      use Ada.Calendar;

   begin --  Time_Image
      return Image_4 (Year (Item)) & "-" & Image_2 (Month (Item)) & "-" &
             Image_2 (Day (Item)) & "-" &
             Image_2 (Integer (Seconds (Item)) / 3600) & "." &
             Image_2 (Integer (Seconds (Item)) / 60 rem 60) & "." &
             Image_2 (Integer (Seconds (Item)) rem 60);
   end Time_Image;

   ---------------------------------------------------------------------------
   --  function Duration_Image:

   function Duration_Image (Time      : in     Duration;
                            Hour      : in     String := "hour";
                            Hours     : in     String := "hours";
                            Minute    : in     String := "minute";
                            Minutes   : in     String := "minutes";
                            Second    : in     String := "second";
                            Seconds   : in     String := "seconds";
                            And_Text  : in     String := "and";
                            None_Text : in     String := "none")
     return String is

      ---------------------------------------------------------------------
      --  function Hour_Image:

      function Hour_Image return String is

         use Ada.Strings;
         use Ada.Strings.Fixed;

         Value : Natural := Natural (Time) / 3_600;

      begin --  Hour_Image
         case Value is
            when 0 =>
               return "";
            when 1 =>
               return "1 " & Hour;
            when others =>
               return Trim (Natural'Image (Value), Both) & " " & Hours;
         end case;
      end Hour_Image;

      ---------------------------------------------------------------------
      --  function Minute_Image:

      function Minute_Image return String is

         use Ada.Strings;
         use Ada.Strings.Fixed;

         Value : Natural := (Natural (Time) / 60) rem 60;

      begin --  Minute_Image
         case Value is
            when 0 =>
               return "";
            when 1 =>
               return "1 " & Minute;
            when others =>
               return Trim (Natural'Image (Value), Both) & " " & Minutes;
         end case;
      end Minute_Image;

      ---------------------------------------------------------------------
      --  function Second_Image:

      function Second_Image return String is

         use Ada.Strings;
         use Ada.Strings.Fixed;

         Value : Natural := Natural (Time) rem 60;

      begin --  Second_Image
         case Value is
            when 0 =>
               return "";
            when 1 =>
               return "1 " & Second;
            when others =>
               return Trim (Natural'Image (Value), Both) & " " & Seconds;
         end case;
      end Second_Image;

      ---------------------------------------------------------------------

   begin --  Duration_Image
      if Hour_Image = "" then
         if Minute_Image = "" then
            if Second_Image = "" then
               return None_Text;
            else
               return Second_Image;
            end if;
         else
            if Second_Image = "" then
               return Minute_Image;
            else
               return Minute_Image & " " & And_Text & " " & Second_Image;
            end if;
         end if;
      else
         if Minute_Image = "" then
            if Second_Image = "" then
               return Hour_Image;
            else
               return Hour_Image & " " & And_Text & " " & Second_Image;
            end if;
         else
            if Second_Image = "" then
               return Hour_Image & " " & And_Text & " " & Minute_Image;
            else
               return Hour_Image & " " & Minute_Image & " " & And_Text & " " &
                        Second_Image;
            end if;
         end if;
      end if;
   end Duration_Image;

   ---------------------------------------------------------------------------

end Kalender;
