------------------------------------------------------------------------------
--
--  procedure Vote (body)
--
--  A CGI program for collecting and counting votes.
--
--  The value of the fields named "subject" and "vote" are registered.
--
--  The program has to have write access to the files named:
--    /home/httpd/cgi-bin/votes/{Subject}/{Vote}/count
--    /home/httpd/cgi-bin/votes/{Subject}/{Vote}/log
--
--  The count files can be used for showing the status using SSI.
--
------------------------------------------------------------------------------
--  Update information:
--
--  1999.09.27 (Jacob Sparre Andersen)
--    Written.
--
------------------------------------------------------------------------------
--  Standard packages:

with Ada.Calendar;
with Ada.Integer_Text_IO;
with Ada.Strings.Unbounded;
with Ada.Text_IO;

------------------------------------------------------------------------------
--  Other packages:

with CGI;
with Kalender;

------------------------------------------------------------------------------

procedure Vote is

   ---------------------------------------------------------------------------
   --  CGI argument names:

   Subject_Argument : constant String := "subject";
   Vote_Argument    : constant String := "vote";
   Thanks_URL       : constant String := "thanks";

   ---------------------------------------------------------------------------
   --  File location:

   Vote_Log_Directory : constant String := "/home/httpd/cgi-bin/votes/";

   ---------------------------------------------------------------------------
   --  function To_Alphanumeric:

   function To_Alphanumeric (Item : in String) return String is

      use Ada.Strings.Unbounded;

      Processed            : Unbounded_String := Null_Unbounded_String;
      Written_Alphanumeric : Boolean := False;

   begin --  To_Alphanumeric
      for Index in Item'Range loop
         case Item (Index) is
            when 'a' .. 'z' | 'A' .. 'Z' | '0' .. '9' =>
               Append (Source   => Processed,
                       New_Item => Item (Index));
               Written_Alphanumeric := True;
            when others =>
               if Written_Alphanumeric then
                  Append (Source   => Processed,
                          New_Item => '_');
                  Written_Alphanumeric := False;
               end if;
         end case;
      end loop;

      if Length (Processed) < 1 then
         raise Constraint_Error;
      else
         return To_String (Processed);
      end if;
   end To_Alphanumeric;

   ---------------------------------------------------------------------------
   --  procedure Count_Vote:

   procedure Count_Vote is

      use Ada.Calendar;
      use Ada.Integer_Text_IO;
      use Ada.Text_IO;
      use CGI;
      use Kalender;

      Subject : constant String := To_Alphanumeric (Value (Subject_Argument));
      Vote    : constant String := To_Alphanumeric (Value (Vote_Argument));

      Remote_Addr : constant String := Get_Environment ("REMOTE_ADDR");

      Vote_Log_File   : File_Type;
      Vote_Count_File : File_Type;
      Vote_Count      : Natural := 0;

   begin --  Count_Vote
      Open (File => Vote_Log_File,
            Name => Vote_Log_Directory & Subject & "/" & Vote & "/log",
            Mode => Append_File);
      Put_Line (File => Vote_Log_File,
                Item => Date_Image (Clock) & " " & Remote_Addr);
      Close (File => Vote_Log_File);

      Open (File => Vote_Log_File,
            Name => Vote_Log_Directory & Subject & "/" & Vote & "/log",
            Mode => In_File);
      while not End_Of_File (File => Vote_Log_File) loop
         Vote_Count := Vote_Count + 1;
         Skip_Line (File => Vote_Log_File);
      end loop;
      Close (File => Vote_Log_File);

      Open (File => Vote_Count_File,
            Name => Vote_Log_Directory & Subject & "/" & Vote & "/count",
            Mode => Out_File);
      Put (File  => Vote_Count_File,
           Item  => Vote_Count,
           Width => 0);
      Close (File => Vote_Count_File);

      if Key_Exists (Thanks_URL) then
         Put_CGI_Header ("Location: " & Value (Thanks_URL));
      else
         Put_CGI_Header;
         Put_HTML_Head (Title => "Thanks for voting.");
         Put_HTML_Heading (Title => "Thanks for voting",
                           Level => 1);
         Put_Line (Item => "<P>Your vote has been counted.");
      end if;
   exception
      when others =>
         Put_CGI_Header;
         Put_Error_Message ("Couldn't register your vote. - Most likely " &
                              "because the item you voted on isn't " &
                              "registered (" & Subject & "/" & Vote & ").");
         Put_Line ("<UL>");
         Put_Line ("<LI>" & Vote_Log_Directory & Subject & "/" & Vote & "/log");
         Put_Line ("<LI>" & Vote_Log_Directory & Subject & "/" & Vote & "/count");
         Put_Line ("</UL>");
         raise;
   end Count_Vote;

   ---------------------------------------------------------------------------

   use CGI;

begin --  Vote
   if Key_Count (Subject_Argument) = 0 then
      Put_CGI_Header;
      Put_Error_Message ("No """ & Subject_Argument & """ argument.");
   elsif Key_Count (Subject_Argument) = 1 then
      if Key_Count (Vote_Argument) = 1 then
         Count_Vote;
      else
         Put_CGI_Header;
         Put_Error_Message ("The program should be passed exactly one """ &
                              Vote_Argument & """ argument.");
      end if;
   else
      Put_CGI_Header;
      Put_Error_Message ("The program can only handle one """ &
                           Subject_Argument & """ argument at a time.");
   end if;
end Vote;
